/* 
 * CSE142, Spring 1999
 * Homework 4
 * Traffic animation

  STARTER PROGRAM

 *  Program by:		Daven Henze	
 *  Section:		BG
 */ 


#include "gp142.h"
#include <stdio.h>
#include <math.h>

#include <time.h>
	/* 
	* <time.h> is included so the current time can be accessed
	* to initialize the random number generator.  If you don't
	* use random numbers (and you don't have to) you can either
	* ignore this #include or delete it.
	*/

/* Logical constants */
#define TRUE    		1
#define FALSE   		0

/* Time Mode constants */
#define DAY				1

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.  The
 * constant FILL is used in function calls when a solid
 * shape is desired.
 */
#define FILL 0


/*
 * Symbolic constants.  Change as needed.
 */

/* Menu buttons: */

/* Top-left coordinates of button palette */
#define MENU_TOP		(GP142_YMAX - 5)
#define MENU_LEFT		(-GP142_XMAX + 5)

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  	25
#define BUTTON_WIDTH 	75

/* Number of buttons */
#define NUM_BUTTONS 	6


/* Possible locations of mouse clicks: */

/*   Mouse click in some button */
#define BUTTON_CLICK 1

/*   Mouse click elsewhere */
#define OTHER_CLICK 0


/* Coordinates of objects in the drawing */

/* y-coordinate of the top of the ground in the landscape */
#define HORIZON (- 2 * (GP142_YMAX / 10))

/* Vertical center and width of highway */
#define HIGHWAY_CENTER ((-GP142_YMAX + HORIZON) / 3)
#define HIGHWAY_WIDTH  10

/* Dimensions for sign and post */
#define POST_BOTTOM		(HIGHWAY_CENTER + HIGHWAY_WIDTH)
#define POST_HEIGHT		20
#define POST_WIDTH		5
#define SIGN_BOTTOM		(POST_BOTTOM + POST_HEIGHT)
#define SIGN_HEIGHT		35
#define SIGN_WIDTH		100

/* Dimensions for speedometer */
#define METER_WIDTH		150
#define METER_HEIGHT	(-GP142_YMAX + 20)

/* Maximum number of vehicles that can appear in the animation */
#define MAX_VEHICLES	1

/* Height and width of a single car */
#define CAR_HEIGHT 24
#define CAR_WIDTH  40

/* Maximum speed that car can travel without being pulled over */
#define MAX_SPEED 10

/* Amount of time that a 'pull over' takes */
#define ARREST_LENGTH 90
/*****************************************************************************

     FUNCTION PROTOTYPES

*****************************************************************************/

/* Initialize random number generator.  [Ignore or		*/
/* remove if your program doesn't use random numbers.]	*/
void initialize_random_numbers();

/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y, int time, int speed, 
					int car_caught, int cloud_x[], int cloud_y[]);

/* Draw the buttons */
void draw_buttons(int time);

/* Return kind of mouse click (in a button, etc.) */
int  classify_mouse_click(int mouse_x, int mouse_y);

/* Process button click at location x, y.  Update car speed,  */
/* direction, color, and whether traffic is moving as needed. */
void handle_button(int x, int y, int *traffic_moving, int *car_speed,
				   int *car_direction, int *cop_direction, 
				   int *car_color, int *cop_call, int *cop_speed);

/* Update sun coordinates sun_x, sun_y for one tick */
void move_sun(int *sun_x, int *sun_y, int *time);
void move_cloud(int cloud_x[]);

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick. */
void move_traffic(int *x, int *y, int *direction, int cop_direction, 
				  int speed, int *xp, int* yp, int *cop_speed, 
				  int *cop_call, int *car_caught, int *arrest_time, 
				  int *car_speed);

/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(int x, int y, int color, int time, int direction,
				  int xp, int yp, int cop_call, int strobe, int arrest_time);

/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color, int time, int direction);

void draw_police_car(int x, int y, int strobe, int arrest_time);
/* Print dialogue during cop interaction */
void arrest_dialogue(int arrest_time);
void alternate_dialogue(int arrest_time);
/* Display error message if internal error is detected. */
void something_is_wrong (void);


/*****************************************************************************

     MAIN

*****************************************************************************/

/*
 * Main program contents:
 *   - Major program variables that persist throughout execution
 *   - Event loop that processes GP142 events until the user quits,
 *     and updates variables and redraws the window as needed.
 */
int main(void)
{
	/* Event handling and buttons: */
	int  quit;				/* = "user has selected quit"				*/
	int  event;				/* the most recent GP142 event				*/
	char key_pressed;		/* last keyboard character from key event	*/
	int  mouse_x, mouse_y;	/* x and y coordinates of latest mouse event*/
	int  click_kind;		/* kind of mouse click (button, etc.)		*/

	/* Information about the sun: */
	int sun_x, sun_y;			/* screen x and y coordinates */
	int cloud_x[]= {-400,-200,0,200};
	int cloud_y[]={180,220,190,200};
	/* Operation mode */
	int time=DAY;
	int cop_call;
	int car_caught;
	int arrest_time;
	
	/* Flashing lights operator */
	int strobe;

	/* Information about vehicles in the animation:							*/
	/* [Only one car now - will need to be changed to add more]				*/
	int car_x, car_y;		/* current position of the car					*/
	int car_direction;		/* current direction: +1=right, -1=left			*/
	int car_color;			/* color of the car								*/
	int car_speed;			/* speed in pixels per tick when traffic moving	*/	
	int traffic_moving;		/* = "traffic is currently moving"				*/
	int cop_x, cop_y;       /* current position of cop car */
	int cop_speed;			/* current speed of cop car */
	int cop_direction;		/* current direction of cop car */

	initialize_random_numbers();
	
	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.			*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* set initial location of the sun */
	sun_x = GP142_XMAX - 50;
	sun_y = GP142_YMAX - 35;

	/* set initial conditions for the car */
	car_x = 0;
	car_y = HIGHWAY_CENTER;
	car_direction = +1;
	car_speed = 5;
	car_color = RED;
	traffic_moving = TRUE;

	/* set initial conditions for cop car */
	cop_x = -GP142_XMAX;
	cop_y = HIGHWAY_CENTER;
	cop_speed = 20;
	cop_call = FALSE;
	car_caught = FALSE;
	arrest_time = 0;
	strobe = 0;
	/* Draw initial landscape, buttons, and traffic */
	draw_landscape(sun_x,sun_y, (time%2), car_speed, car_caught,
		cloud_x, cloud_y);
	draw_buttons(time%2);
	draw_traffic(car_x, car_y, car_color, time%2, car_direction,
				cop_x, cop_y, cop_call, strobe, arrest_time);
	

	/*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */
	quit = FALSE;
	while (!quit) {
		
		/* get next event */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			/* Key pressed.  Not used now.  If the program should		*/
			/* react to keyboard events, insert something appropriate	*/
			/* here.  Note that scanf CANNOT be used with GP142.		*/
			break;
			
		case GP142_MOUSE:
			/* Mouse click.  Determine if in a menu button	*/
			/* or elsewhere and process accordingly.		*/
			click_kind = classify_mouse_click(mouse_x, mouse_y);
			if (click_kind == BUTTON_CLICK) {
				handle_button(mouse_x, mouse_y, &traffic_moving, &car_speed,
								&car_direction, &cop_direction, &car_color,
								&cop_call, &cop_speed);
			} else {
				/* insert code here (and additional else-if statements) */
				/* to react to mouse clicks elsewhere on the screen */
			}
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances.	*/
			strobe = strobe +1;
			move_sun(&sun_x, &sun_y, &time);
			move_cloud(cloud_x);
			if (traffic_moving)
				move_traffic(&car_x, &car_y, &car_direction, cop_direction,
				car_speed, &cop_x, &cop_y, &cop_speed, &cop_call, &car_caught, 
				&arrest_time, &car_speed);
			/* stop traffic when cop catches car */
			if (cop_call){
				if(cop_direction == 1){
					if ((cop_x > car_x - 55) && (cop_x < car_x - 25))	
					car_caught = TRUE;
					}
				else {
					if ((cop_x < car_x + 55) && (cop_x > car_x + 25))	
					car_caught = TRUE;
					}
				}
			
			break;

			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */
		
		/* Redraw everything */
		draw_landscape(sun_x, sun_y, time%2, car_speed, car_caught,
			cloud_x, cloud_y);
		draw_buttons(time%2);
		draw_traffic(car_x, car_y, car_color, time%2, car_direction,
					cop_x, cop_y, cop_call, strobe, arrest_time);
		if (car_caught){
			if (car_x%4==0) alternate_dialogue(arrest_time);
			else arrest_dialogue(arrest_time);}
	}  /* end event loop */
	
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return 0;
}  /*end main */


/*****************************************************************************

     BACKGROUND LANDSCAPE

*****************************************************************************/


/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y, int time, int speed, int car_caught,
					int cloud_x[], int cloud_y[])
{
	int text_offset = -GP142_XMAX + 15;	/* horizontal offset of text */
	int text_size   = 12;				/* size of text font */
	int c_offset	= 10;				/* distance between cloud circles */
	int c_radious	= 20;				/* radious of cloud circles */
	/* Draw the sky */
	if(time==DAY){ /*Draw dayscape*/
		GP142_rectangleXY(ICE_BLUE, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);
	 
	/* Draw the sun */
	
		GP142_circleXY(YELLOW, sun_x, sun_y , 20);
	}
	else {  /*draw nightscape*/
		GP142_rectangleXY(BLACK, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);
	 
	/* Draw the sun */
	
		GP142_circleXY(WHITE, sun_x, sun_y , 20); }
	
	
	/* Draw the ground, highway, speed sign and text legend */
	GP142_rectangleXY(BROWN, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, HORIZON, FILL);
	GP142_rectangleXY(WHITE, 
					-GP142_XMAX, HIGHWAY_CENTER - HIGHWAY_WIDTH,
					 GP142_XMAX, HIGHWAY_CENTER + HIGHWAY_WIDTH,
					 FILL);
	GP142_lineXY(BLACK, -GP142_XMAX, HIGHWAY_CENTER,
					GP142_XMAX, HIGHWAY_CENTER, FILL);
	GP142_rectangleXY(BLACK, -POST_WIDTH/2, POST_BOTTOM,
						POST_WIDTH/2, POST_BOTTOM + POST_HEIGHT, FILL);
	GP142_rectangleXY(ORANGE, -SIGN_WIDTH/2, SIGN_BOTTOM, SIGN_WIDTH/2,
					SIGN_BOTTOM+SIGN_HEIGHT, FILL);
	GP142_printfXY(BLACK, -SIGN_WIDTH/2 +4, SIGN_BOTTOM 
		+ SIGN_HEIGHT/6, text_size, "SPEED LIMIT 100");
	GP142_printfXY (BLACK, text_offset, -GP142_YMAX+30, text_size,
		"Click a button to control the car.");
	
	/* Draw speedometer */
	if (car_caught) speed = 0;
	GP142_rectangleXY(BLACK, -METER_WIDTH/2, -GP142_YMAX, METER_WIDTH/2, 
					METER_HEIGHT, FILL); 
	GP142_printfXY(RED, -METER_WIDTH/2 +5, -GP142_YMAX + 5, text_size,
		"CURRENT SPEED: %dmph", speed*10);
	
	/*Draw some mountains */
	GP142_triangleXY(MED_GRAY,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	
	GP142_triangleXY(OLIVE,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	
	GP142_triangleXY(FOREST_GREEN,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL);
	
	/* Draw Clouds */
		GP142_circleXY(WHITE, cloud_x[0], cloud_y[0], c_radious);
	GP142_circleXY(WHITE, cloud_x[0] +c_offset, cloud_y[0]+c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[0] +2*c_offset, cloud_y[0]-c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[0] +3*c_offset, cloud_y[0]+c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[0] +4*c_offset, cloud_y[0]-c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[0] +5*c_offset, cloud_y[0], c_radious);
	GP142_circleXY(WHITE, cloud_x[1], cloud_y[1], c_radious);
	GP142_circleXY(WHITE, cloud_x[1] +c_offset, cloud_y[1]+c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[1] +2*c_offset, cloud_y[1]-c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[1] +3*c_offset, cloud_y[1]+c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[1] +4*c_offset, cloud_y[1]-c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[1] +5*c_offset, cloud_y[1], c_radious);
	GP142_circleXY(WHITE, cloud_x[2], cloud_y[2], c_radious);
	GP142_circleXY(WHITE, cloud_x[2] +c_offset, cloud_y[2]+c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[2] +2*c_offset, cloud_y[2]-c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[2] +3*c_offset, cloud_y[2]+c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[2] +4*c_offset, cloud_y[2]-c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[2] +5*c_offset, cloud_y[2], c_radious);
	GP142_circleXY(WHITE, cloud_x[3], cloud_y[3], c_radious);
	GP142_circleXY(WHITE, cloud_x[3] +c_offset, cloud_y[3]+c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[3] +2*c_offset, cloud_y[3]-c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[3] +3*c_offset, cloud_y[3]+c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[3] +4*c_offset, cloud_y[3]-c_offset, c_radious);
	GP142_circleXY(WHITE, cloud_x[3] +5*c_offset, cloud_y[3], c_radious);
} /*end draw_landscape */



/*****************************************************************************

     BUTTONS AND MOUSE CLICKS

*****************************************************************************/


/* Draws the buttons */
void draw_buttons(int time)
{
	int i;
	int text_offset = MENU_LEFT+10;	/* offset of button text in button */
	int font_size = 12;				/* size of button text */
	
	/* Daytime conditions */
	if(time==DAY) {

	/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(BLACK,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(FOREST_GREEN, text_offset, i, font_size, "Go");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(RED, text_offset, i, font_size, "Stop");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Faster");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Slower");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Reverse");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "New Color");
	}

	/*Draw Night conditions */
	else {/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(WHITE,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(FOREST_GREEN, text_offset, i, font_size, "Go");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(RED, text_offset, i, font_size, "Stop");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(WHITE, text_offset, i, font_size, "Faster");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(WHITE, text_offset, i, font_size, "Slower");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(WHITE, text_offset, i, font_size, "Reverse");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(WHITE, text_offset, i, font_size, "New Color");
	}
}

/* Classify mouse click at (mouse_x,mouse_y) and return kind */
int  classify_mouse_click(int mouse_x, int mouse_y)
{
	if ((mouse_x > MENU_LEFT) &&
		(mouse_x < MENU_LEFT + BUTTON_WIDTH) &&
		(mouse_y < MENU_TOP) &&
		(mouse_y > (MENU_TOP - NUM_BUTTONS*BUTTON_HEIGHT)))
		return BUTTON_CLICK;
	else
		return OTHER_CLICK;
}



/* Process button click at location x, y.  Update car speed,	*/
/* direction, color, and whether traffic is moving as needed.	*/
void handle_button(int x, int y, int *traffic_moving, int *car_speed,
				   int *car_direction, int *cop_direction, 
				   int *car_color, int *cop_call, int *cop_speed)
{
	int buttonNum;		/* number of selected button */

	/* Complain if x coordinate couldn't be inside a button */
	if (x < MENU_LEFT || x > MENU_LEFT + BUTTON_WIDTH) {
		something_is_wrong();
		return;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	case 0:
		/* go */
		*traffic_moving = TRUE;
		break;
	case 1:
		/* stop */
		*traffic_moving = FALSE;
		break;
		
	case 2:
		/* speed up */
		*car_speed = *car_speed + 1;
		if(*car_speed > MAX_SPEED){
			*cop_call = TRUE;
			*cop_direction = *car_direction;
			*cop_speed = *cop_speed + 2;}
		break;
	case 3:
		/* slow down */
		*car_speed = *car_speed - 1;
		if (*car_speed < 0)
			*car_speed = 0;
		
		break;
		
	case 4:
		/* reverse */
		*car_direction = - *car_direction;
		break;
	case 5:
		/* change car color */
		*car_color = (*car_color+1) % MAX_COLORS;
		break;
	default:
		/*Bad news if this is ever reached! */
		something_is_wrong();
		break;
	}
}



/*****************************************************************************

     TRAFFIC AND OTHER ANIMATED OBJECTS

*****************************************************************************/


/* Update sun coordinates sun_x, sun_y by one tick */
void move_sun(int *sun_x, int *sun_y, int *time) {
	/* Move the sun to the left.  If it reaches the edge, move it */
	/* to the other side and down a bit. */
	*sun_x=*sun_x - 3;
	if (*sun_x < -GP142_XMAX -30) {
		*sun_x = - *sun_x;
		(*sun_y) --;
		*time +=1;
	}
}

/* Update cloud coordinates */
void move_cloud(int cloud_x[]){
	int i;
	for(i=0; i<4; i++){
		cloud_x[i] += 1;
	if (cloud_x[i] > GP142_XMAX)
		cloud_x[i] = - cloud_x[i]-90;}
}



/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick, */
void move_traffic(int *x, int *y, int *direction, int cop_direction, 
				  int speed, int *xp, int *xy, int *cop_speed, int *cop_call,
				  int *car_caught, int *arrest_time, int *car_speed) {
	/* Note: If there is more than one vehicle, this should */
	/* be updated to adjust all vehicle coordinates */

	/* new value for x coordinate */
	int trial_x;
	
	/* Don't update if car has been caught by cop */
	if (!*car_caught){
			

	/* move cop car if cop has been called */
	if (*cop_call) {
		trial_x = *xp + (cop_direction *(* cop_speed));
	if (abs(trial_x) > GP142_XMAX + CAR_WIDTH/2) 
		*xp = -*xp;
	else
		*xp = trial_x;
	}

	/* move car.  If it moves off the edge, wrap around */
		
		trial_x = *x + (*direction * speed);
		if (abs(trial_x) > GP142_XMAX + CAR_WIDTH/2) 
		*x = -*x;
		else
		*x = trial_x;
		}
	
	/* Pause animation while car gets ticket */
	if (*car_caught) 
		*arrest_time = *arrest_time + 1;
	/* Move cop car off screen after arrest time passes */
	if (*arrest_time > ARREST_LENGTH) {
		trial_x = *xp + (cop_direction * (*cop_speed));
		if (abs(trial_x) > GP142_XMAX + CAR_WIDTH/2) {
	/* Reset to initial conditions */	
		*car_caught = FALSE;
		*cop_call=FALSE;
		*arrest_time = 0;
		*car_speed = 5;
		*cop_speed = 20;}
	else
		*xp = trial_x;

	}
}
	
/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(int x, int y, int color, int time, int direction, 
				  int xp, int yp, int cop_call, int strobe, int arrest_time) {
	/* Note: needs to be changed if there is more than */
	/* one vehicle */
	if (cop_call) draw_police_car(xp, yp, strobe, arrest_time);
	draw_car(x, y, color, time, direction);
}



/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color, int time, int direction) {
	GP142_rectangleXY(color, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x + CAR_WIDTH/2, y + CAR_HEIGHT*3/4, FILL);
	GP142_rectangleXY(color, x - CAR_WIDTH/6, y + CAR_HEIGHT*3/4,
						x + CAR_WIDTH/6, y + CAR_HEIGHT, FILL);
	GP142_circleXY(BLACK, x - CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_circleXY(BLACK, x + CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	/* Draw headlights at night */
	if (time!=DAY) {
		if(direction== -1){
			GP142_triangleXY(YELLOW, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x-CAR_WIDTH*2, y, 
						x-CAR_WIDTH*2, y + CAR_HEIGHT/2, FILL);
			}		
		else {
			GP142_triangleXY(YELLOW, x + CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x+CAR_WIDTH*2, y, 
						x+CAR_WIDTH*2, y + CAR_HEIGHT/2, FILL);
			}
	}
}

/* Draw Police Car given center xp at base  level yp  */
void draw_police_car(int x, int y, int strobe, int arrest_time) {
	int light_1, light_2;
	/* Flash cop lights between red and blue */
	if(strobe%2) {
		light_1 = RED;
		light_2 = BLUE;}
	else { 
		light_2 = RED; 
		light_1 = BLUE;}
	
	/* Draw main frame */
	GP142_rectangleXY(BLACK, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x + CAR_WIDTH/2, y + CAR_HEIGHT*3/4, FILL);
	GP142_rectangleXY(WHITE, x-CAR_WIDTH/6, y + CAR_HEIGHT/4,
					x+CAR_WIDTH/6, y + CAR_HEIGHT*3/4, FILL);
	GP142_rectangleXY(BLACK, x - CAR_WIDTH/6, y + CAR_HEIGHT*3/4,
						x + CAR_WIDTH/6, y + CAR_HEIGHT, FILL);
	GP142_circleXY(BLACK, x - CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_circleXY(BLACK, x + CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_rectangleXY(BLACK, x - CAR_WIDTH/14, y + CAR_HEIGHT,
					x + CAR_WIDTH/14, y + CAR_HEIGHT +6, FILL);
	GP142_lineXY(BLACK, x - CAR_WIDTH/6, y + CAR_HEIGHT/4,
				x + CAR_WIDTH/6, y + CAR_HEIGHT/4, FILL);
	/* Draw lights */ 
	if (arrest_time < 45) {
		GP142_triangleXY(light_1, x - CAR_WIDTH/14, y + CAR_HEIGHT +3,
					x - 60, y + CAR_HEIGHT,
					x - 60, y + CAR_HEIGHT + 6, FILL);
	GP142_triangleXY(light_2, x + CAR_WIDTH/14, y + CAR_HEIGHT +3,
					x + 60, y + CAR_HEIGHT,
					x + 60, y + CAR_HEIGHT + 6, FILL);
	}
}


/*****************************************************************************

     OTHER

*****************************************************************************/

/* Display dialogue for cop interaction */
void arrest_dialogue(int arrest_time){
	int text_large = 30,
		text_small = 25,
		left =		-250,
		bottom =	-GP142_YMAX*2/3;
	
	if (arrest_time <15){
		GP142_printfXY(BLACK, left, bottom,
			text_large, "COP: Do you know how fast you were going?");}
	else if(arrest_time <30) {
		GP142_printfXY(BLACK, left, bottom,
			text_large, "It's going to be a $75 ticket.");}
	else if(arrest_time < 45) {
		GP142_printfXY(BLACK, left, bottom,
			text_large, "You can make that a check payable to:");}
	else {GP142_printfXY(NAVY_BLUE, -100, bottom,
			text_small, "Daven Henze \n4235 12th Ave NE \nSeattle, WA 98105");}
	}

/* Display alternate dialogue for cop interaction */
void alternate_dialogue(int arrest_time){
	int text_large = 30,
		text_small = 25,
		left =		-250,
		bottom =	-GP142_YMAX*2/3;
	if (arrest_time <15){
		GP142_printfXY(BLACK, left-30, bottom,
			text_large, "COP: Do you know how fast you were going?");}
	else if(arrest_time <40) {
		GP142_printfXY(BLACK, left, bottom,
			text_large, "Your speeding ticket's going to be --\n    Hey, is that reefer I smell?...");}
	else if(arrest_time < 65) {
		GP142_printfXY(BLACK, left, bottom,
			text_large, "Sorry but I'm going to have to confiscate that.");}
	else {GP142_printfXY(BLACK, -100, bottom,
			text_large, "Thank you, have a nice day.");}
	}

/* Initialize random number generator */
/* [Ignore or delete if your program  */
/* doesn't user random numbers.] */
void initialize_random_numbers() {
	time_t t;			/* current time */
	srand (time(&t));
}



/* Display error message if internal error is detected. */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}